@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2023 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 /* PDFObject interface */
 
@@ -455,7 +455,7 @@ FUN(PDFObject_getArray)(JNIEnv *env, jobject self, jint index)
 }
 
 JNIEXPORT jobject JNICALL
-FUN(PDFObject_getDictionary)(JNIEnv *env, jobject self, jstring jname)
+FUN(PDFObject_getDictionary)(JNIEnv *env, jobject self, jstring jname, jboolean inheritable)
 {
 	fz_context *ctx = get_context(env);
 	pdf_obj *dict = from_PDFObject(env, self);
@@ -471,7 +471,10 @@ FUN(PDFObject_getDictionary)(JNIEnv *env, jobject self, jstring jname)
 	if (!name) jni_throw_run(env, "cannot get name to lookup");
 
 	fz_try(ctx)
-		val = pdf_dict_gets(ctx, dict, name);
+		if (inheritable)
+			val = pdf_dict_gets_inheritable(ctx, dict, name);
+		else
+			val = pdf_dict_gets(ctx, dict, name);
 	fz_always(ctx)
 		(*env)->ReleaseStringUTFChars(env, jname, name);
 	fz_catch(ctx)
@@ -558,7 +561,7 @@ FUN(PDFObject_putArrayString)(JNIEnv *env, jobject self, jint index, jstring jst
 	fz_try(ctx)
 	{
 		if (str)
-			pdf_array_put_drop(ctx, arr, index, pdf_new_string(ctx, str, strlen(str)));
+			pdf_array_put_string(ctx, arr, index, str, strlen(str));
 		else
 			pdf_array_put(ctx, arr, index, PDF_NULL);
 	}
@@ -901,6 +904,105 @@ FUN(PDFObject_putDictionaryPDFObjectDate)(JNIEnv *env, jobject self, jobject jna
 		jni_rethrow_void(env, ctx);
 }
 
+JNIEXPORT void JNICALL
+FUN(PDFObject_putDictionaryStringRect)(JNIEnv *env, jobject self, jstring jname, jobject jrect)
+{
+	fz_context *ctx = get_context(env);
+	pdf_obj *dict = from_PDFObject(env, self);
+	fz_rect rect  = from_Rect(env, jrect);
+	const char *name = NULL;
+	pdf_obj *key = NULL;
+
+	if (!ctx || !dict) return;
+	if (jname)
+	{
+		name = (*env)->GetStringUTFChars(env, jname, NULL);
+		if (!name) return;
+	}
+
+	fz_var(key);
+
+	fz_try(ctx)
+	{
+		key = name ? pdf_new_name(ctx, name) : NULL;
+		pdf_dict_put_rect(ctx, dict, key, rect);
+	}
+	fz_always(ctx)
+	{
+		pdf_drop_obj(ctx, key);
+		if (name)
+			(*env)->ReleaseStringUTFChars(env, jname, name);
+	}
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFObject_putDictionaryStringMatrix)(JNIEnv *env, jobject self, jstring jname, jobject jmatrix)
+{
+	fz_context *ctx = get_context(env);
+	pdf_obj *dict = from_PDFObject(env, self);
+	fz_matrix matrix  = from_Matrix(env, jmatrix);
+	const char *name = NULL;
+	pdf_obj *key = NULL;
+
+	if (!ctx || !dict) return;
+	if (jname)
+	{
+		name = (*env)->GetStringUTFChars(env, jname, NULL);
+		if (!name) return;
+	}
+
+	fz_var(key);
+
+	fz_try(ctx)
+	{
+		key = name ? pdf_new_name(ctx, name) : NULL;
+		pdf_dict_put_matrix(ctx, dict, key, matrix);
+	}
+	fz_always(ctx)
+	{
+		pdf_drop_obj(ctx, key);
+		if (name)
+			(*env)->ReleaseStringUTFChars(env, jname, name);
+	}
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFObject_putDictionaryStringDate)(JNIEnv *env, jobject self, jstring jname, jlong time)
+{
+	fz_context *ctx = get_context(env);
+	pdf_obj *dict = from_PDFObject(env, self);
+	const char *name = NULL;
+	pdf_obj *key = NULL;
+
+	if (!ctx || !dict) return;
+	if (jname)
+	{
+		name = (*env)->GetStringUTFChars(env, jname, NULL);
+		if (!name) return;
+	}
+
+	fz_var(key);
+
+	fz_try(ctx)
+	{
+		key = name ? pdf_new_name(ctx, name) : NULL;
+		pdf_dict_put_date(ctx, dict, key, time);
+	}
+	fz_always(ctx)
+	{
+		pdf_drop_obj(ctx, key);
+		if (name)
+			(*env)->ReleaseStringUTFChars(env, jname, name);
+	}
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+
 JNIEXPORT void JNICALL
 FUN(PDFObject_deleteArray)(JNIEnv *env, jobject self, jint index)
 {
@@ -1253,3 +1355,20 @@ FUN(PDFObject_equals)(JNIEnv *env, jobject self, jobject jother)
 
 	return result == 0 ? JNI_TRUE : JNI_FALSE;
 }
+
+JNIEXPORT jboolean JNICALL
+FUN(PDFObject_isFileSpec)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_obj *obj = from_PDFObject_safe(env, self);
+	int result = 0;
+
+	if (!ctx) return JNI_FALSE;
+
+	fz_try(ctx)
+		result = pdf_is_filespec(ctx, obj);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return result ? JNI_TRUE : JNI_FALSE;
+}
